@@ -141,7 +141,9 @@ binary_quote(binaryObject *self)
     /* if we got a plain string or a buffer we escape it and save the buffer */
     if (PyString_Check(self->wrapped) || PyBuffer_Check(self->wrapped)) {
         /* escape and build quoted buffer */
-        PyObject_AsCharBuffer(self->wrapped, &buffer, &buffer_len);
+        if (PyObject_AsReadBuffer(self->wrapped, (const void **)&buffer,
+                                  &buffer_len) < 0)
+            return NULL;
 
         to = (char *)binary_escape((unsigned char*)buffer, (size_t) buffer_len,
             &len, self->conn ? ((connectionObject*)self->conn)->pgconn : NULL);
@@ -150,10 +152,13 @@ binary_quote(binaryObject *self)
             return NULL;
         }
 
-    if (len > 0)
-        self->buffer = PyString_FromFormat("'%s'", to);
-    else
-        self->buffer = PyString_FromString("''");
+        if (len > 0)
+            self->buffer = PyString_FromFormat(
+                (self->conn && ((connectionObject*)self->conn)->equote)
+                    ? "E'%s'" : "'%s'" , to);
+        else
+            self->buffer = PyString_FromString("''");
+
         PQfreemem(to);
     }
 
@@ -178,14 +183,14 @@ binary_str(binaryObject *self)
     return self->buffer;
 }
 
-PyObject *
+static PyObject *
 binary_getquoted(binaryObject *self, PyObject *args)
 {
     if (!PyArg_ParseTuple(args, "")) return NULL;
     return binary_str(self);
 }
 
-PyObject *
+static PyObject *
 binary_prepare(binaryObject *self, PyObject *args)
 {
     connectionObject *conn;
@@ -203,7 +208,7 @@ binary_prepare(binaryObject *self, PyObject *args)
     return Py_None;
 }
 
-PyObject *
+static PyObject *
 binary_conform(binaryObject *self, PyObject *args)
 {
     PyObject *res, *proto;
