@@ -38,14 +38,14 @@
 
 /* the pointer to the datetime module API is initialized by the module init
    code, we just need to grab it */
-extern PyObject* pyDateTimeModuleP;
-extern PyObject *pyDateTypeP;
-extern PyObject *pyTimeTypeP;
-extern PyObject *pyDateTimeTypeP;
-extern PyObject *pyDeltaTypeP;
+extern HIDDEN PyObject* pyDateTimeModuleP;
+extern HIDDEN PyObject *pyDateTypeP;
+extern HIDDEN PyObject *pyTimeTypeP;
+extern HIDDEN PyObject *pyDateTimeTypeP;
+extern HIDDEN PyObject *pyDeltaTypeP;
 
-extern PyObject *pyPsycopgTzModule;
-extern PyObject *pyPsycopgTzLOCAL;
+extern HIDDEN PyObject *pyPsycopgTzModule;
+extern HIDDEN PyObject *pyPsycopgTzLOCAL;
 
 /* datetime_str, datetime_getquoted - return result of quoting */
 
@@ -64,7 +64,7 @@ pydatetime_str(pydatetimeObject *self)
     else {
         PyDateTime_Delta *obj = (PyDateTime_Delta*)self->wrapped;
 
-        char buffer[8];
+        char buffer[8]; 
         int i;
         int a = obj->microseconds;
 
@@ -79,14 +79,14 @@ pydatetime_str(pydatetimeObject *self)
     }
 }
 
-PyObject *
+static PyObject *
 pydatetime_getquoted(pydatetimeObject *self, PyObject *args)
 {
     if (!PyArg_ParseTuple(args, "")) return NULL;
     return pydatetime_str(self);
 }
 
-PyObject *
+static PyObject *
 pydatetime_conform(pydatetimeObject *self, PyObject *args)
 {
     PyObject *res, *proto;
@@ -128,8 +128,7 @@ pydatetime_setup(pydatetimeObject *self, PyObject *obj, int type)
 {
     Dprintf("pydatetime_setup: init datetime object at %p, refcnt = "
         FORMAT_CODE_PY_SSIZE_T,
-        self, ((PyObject *)self)->ob_refcnt
-      );
+        self, ((PyObject *)self)->ob_refcnt);
 
     self->type = type;
     self->wrapped = obj;
@@ -137,8 +136,7 @@ pydatetime_setup(pydatetimeObject *self, PyObject *obj, int type)
 
     Dprintf("pydatetime_setup: good pydatetime object at %p, refcnt = "
         FORMAT_CODE_PY_SSIZE_T,
-        self, ((PyObject *)self)->ob_refcnt
-      );
+        self, ((PyObject *)self)->ob_refcnt);
     return 0;
 }
 
@@ -402,22 +400,26 @@ psyco_TimestampFromTicks(PyObject *self, PyObject *args)
     time_t t;
     double ticks;
 
-    if (!PyArg_ParseTuple(args,"d", &ticks))
+    if (!PyArg_ParseTuple(args, "d", &ticks))
         return NULL;
 
     t = (time_t)round(ticks);
     ticks -= (double)t;
     if (localtime_r(&t, &tm)) {
-        args = Py_BuildValue("iiiiidO",
-                             tm.tm_year+1900, tm.tm_mon+1, tm.tm_mday,
-                             tm.tm_hour, tm.tm_min,
-                 (double)tm.tm_sec + ticks,
-                             pyPsycopgTzLOCAL);
-        if (args) {
-            res = psyco_Timestamp(self, args);
-            Py_DECREF(args);
+        PyObject *value = Py_BuildValue("iiiiidO",
+            tm.tm_year+1900, tm.tm_mon+1, tm.tm_mday,
+            tm.tm_hour, tm.tm_min,
+            (double)tm.tm_sec + ticks,
+            pyPsycopgTzLOCAL);
+        if (value) {
+            /* FIXME: not decref'ing the value here is a memory leak
+	       but, on the other hand, if we decref we get a clean nice
+	       segfault (on my 64 bit Python 2.4 box). So this leaks
+	       will stay until after 2.0.7 when we'll try to plug it */
+            res = psyco_Timestamp(self, value);
         }
     }
+    
     return res;
 }
 
