@@ -94,6 +94,7 @@ qstring_quote(qstringObject *self)
     PyObject *str;
     char *s, *buffer;
     Py_ssize_t len;
+    int equote;         /* buffer offset if E'' quotes are needed */
 
     /* if the wrapped object is an unicode object we can encode it to match
        self->encoding but if the encoding is not specified we don't know what
@@ -109,7 +110,7 @@ qstring_quote(qstringObject *self)
         /* note that enc is a borrowed reference */
 
         if (enc) {
-            char *s = PyString_AsString(enc);
+            const char *s = PyString_AsString(enc);
             Dprintf("qstring_quote: encoding unicode object to %s", s);
             str = PyUnicode_AsEncodedString(self->wrapped, s, NULL);
             Dprintf("qstring_quote: got encoded object at %p", str);
@@ -141,20 +142,22 @@ qstring_quote(qstringObject *self)
     /* encode the string into buffer */
     PyString_AsStringAndSize(str, &s, &len);
 
-    buffer = (char *)PyMem_Malloc((len*2+3) * sizeof(char));
+    buffer = (char *)PyMem_Malloc((len*2+4) * sizeof(char));
     if (buffer == NULL) {
         Py_DECREF(str);
         PyErr_NoMemory();
         return NULL;
     }
 
+    equote = (self->conn && ((connectionObject*)self->conn)->equote) ? 1 : 0;
+
     { /* Call qstring_escape with the GIL released, then reacquire the GIL
        * before verifying that the results can fit into a Python string; raise
        * an exception if not. */
         size_t qstring_res;
 
         Py_BEGIN_ALLOW_THREADS
-        qstring_res = qstring_escape(buffer+1, s, len,
+        qstring_res = qstring_escape(buffer+equote+1, s, len,
           self->conn ? ((connectionObject*)self->conn)->pgconn : NULL);
         Py_END_ALLOW_THREADS
 
@@ -166,10 +169,12 @@ qstring_quote(qstringObject *self)
             return NULL;
         }
         len = (Py_ssize_t) qstring_res;
-        buffer[0] = '\'' ; buffer[len+1] = '\'';
+        if (equote)
+            buffer[0] = 'E';
+        buffer[equote] = '\'' ; buffer[len+equote+1] = '\'';
     }
 
-    self->buffer = PyString_FromStringAndSize(buffer, len+2);
+    self->buffer = PyString_FromStringAndSize(buffer, len+equote+2);
     PyMem_Free(buffer);
     Py_DECREF(str);
 
@@ -188,14 +193,14 @@ qstring_str(qstringObject *self)
     return self->buffer;
 }
 
-PyObject *
+static PyObject *
 qstring_getquoted(qstringObject *self, PyObject *args)
 {
     if (!PyArg_ParseTuple(args, "")) return NULL;
     return qstring_str(self);
 }
 
-PyObject *
+static PyObject *
 qstring_prepare(qstringObject *self, PyObject *args)
 {
     connectionObject *conn;
@@ -221,7 +226,7 @@ qstring_prepare(qstringObject *self, PyObject *args)
     return Py_None;
 }
 
-PyObject *
+static PyObject *
 qstring_conform(qstringObject *self, PyObject *args)
 {
     PyObject *res, *proto;
@@ -262,7 +267,7 @@ static PyMethodDef qstringObject_methods[] = {
 /* initialization and finalization methods */
 
 static int
-qstring_setup(qstringObject *self, PyObject *str, char *enc)
+qstring_setup(qstringObject *self, PyObject *str, const char *enc)
 {
     Dprintf("qstring_setup: init qstring object at %p, refcnt = "
         FORMAT_CODE_PY_SSIZE_T,
@@ -308,7 +313,7 @@ static int
 qstring_init(PyObject *obj, PyObject *args, PyObject *kwds)
 {
     PyObject *str;
-    char *enc = "latin-1"; /* default encoding as in Python */
+    const char *enc = "latin-1"; /* default encoding as in Python */
 
     if (!PyArg_ParseTuple(args, "O|s", &str, &enc))
         return -1;
@@ -408,7 +413,7 @@ PyObject *
 psyco_QuotedString(PyObject *module, PyObject *args)
 {
     PyObject *str;
-    char *enc = "latin-1"; /* default encoding as in Python */
+    const char *enc = "latin-1"; /* default encoding as in Python */
 
     if (!PyArg_ParseTuple(args, "O|s", &str, &enc))
         return NULL;
