@@ -22,13 +22,25 @@
 #ifndef PSYCOPG_CONFIG_H
 #define PSYCOPG_CONFIG_H 1
 
+/* GCC 4.0 and later have support for specifying symbol visibility */
+#if __GNUC__ >= 4
+#  define HIDDEN __attribute__((visibility("hidden")))
+#else
+#  define HIDDEN
+#endif
+
 /* debug printf-like function */
+#ifdef PSYCOPG_DEBUG
+extern HIDDEN int psycopg_debug_enabled;
+#endif
+
 #if defined( __GNUC__) && !defined(__APPLE__)
 #ifdef PSYCOPG_DEBUG
 #include <sys/types.h>
 #include <unistd.h>
 #define Dprintf(fmt, args...) \
-    fprintf(stderr, "[%d] " fmt "\n", (int) getpid() , ## args)
+    if (!psycopg_debug_enabled) ; else \
+        fprintf(stderr, "[%d] " fmt "\n", (int) getpid() , ## args)
 #else
 #define Dprintf(fmt, args...)
 #endif
@@ -38,6 +50,10 @@
 static void Dprintf(const char *fmt, ...)
 {
     va_list ap;
+
+    if (!psycopg_debug_enabled)
+      return;
+    printf("[%d] ", (int) getpid());
     va_start(ap, fmt);
     vprintf(fmt, ap);
     va_end(ap);
@@ -66,8 +82,8 @@ static void Dprintf(const char *fmt, ...) {}
 #include <winsock2.h>
 #define pthread_mutex_t HANDLE
 #define pthread_condvar_t HANDLE
-#define pthread_mutex_lock(object) WaitForSingleObject(object, INFINITE)
-#define pthread_mutex_unlock(object) ReleaseMutex(object)
+#define pthread_mutex_lock(object) WaitForSingleObject(*(object), INFINITE)
+#define pthread_mutex_unlock(object) ReleaseMutex(*(object))
 #define pthread_mutex_destroy(ref) (CloseHandle(*(ref)))
 /* convert pthread mutex to native mutex */
 static int pthread_mutex_init(pthread_mutex_t *mutex, void* fake)
