@@ -26,6 +26,8 @@
 #include <Python.h>
 #include <libpq-fe.h>
 
+#include "psycopg/config.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -36,7 +38,10 @@ extern "C" {
 #define CONN_STATUS_SYNC  3
 #define CONN_STATUS_ASYNC 4
 
-extern PyTypeObject connectionType;
+/* Hard limit on the notices stored by the Python connection */
+#define CONN_NOTICES_LIMIT 50
+
+extern HIDDEN PyTypeObject connectionType;
 
 typedef struct {
     PyObject_HEAD
@@ -64,31 +69,21 @@ typedef struct {
     /* notifies */
     PyObject *notifies;
 
-    /* errors (DBAPI-2.0 extension) */
-    PyObject *exc_Error;
-    PyObject *exc_Warning;
-    PyObject *exc_InterfaceError;
-    PyObject *exc_DatabaseError;
-    PyObject *exc_InternalError;
-    PyObject *exc_OperationalError;
-    PyObject *exc_ProgrammingError;
-    PyObject *exc_IntegrityError;
-    PyObject *exc_DataError;
-    PyObject *exc_NotSupportedError;
-
     /* per-connection typecasters */
     PyObject *string_types;   /* a set of typecasters for string types */
     PyObject *binary_types;   /* a set of typecasters for binary types */
 
+    int equote;               /* use E''-style quotes for escaped strings */
+
 } connectionObject;
 
 /* C-callable functions in connection_int.c and connection_ext.c */
-extern int  conn_connect(connectionObject *self);
-extern void conn_close(connectionObject *self);
-extern int  conn_commit(connectionObject *self);
-extern int  conn_rollback(connectionObject *self);
-extern int  conn_switch_isolation_level(connectionObject *self, int level);
-extern int  conn_set_client_encoding(connectionObject *self, char *enc);
+HIDDEN int  conn_connect(connectionObject *self);
+HIDDEN void conn_close(connectionObject *self);
+HIDDEN int  conn_commit(connectionObject *self);
+HIDDEN int  conn_rollback(connectionObject *self);
+HIDDEN int  conn_switch_isolation_level(connectionObject *self, int level);
+HIDDEN int  conn_set_client_encoding(connectionObject *self, const char *enc);
 
 /* exception-raising macros */
 #define EXC_IF_CONN_CLOSED(self) if ((self)->closed > 0) { \
