@@ -49,7 +49,7 @@
 static PyObject *
 psyco_conn_cursor(connectionObject *self, PyObject *args, PyObject *keywds)
 {
-    char *name = NULL;
+    const char *name = NULL;
     PyObject *obj, *factory = NULL;
 
     static char *kwlist[] = {"name", "cursor_factory", NULL};
@@ -117,11 +117,8 @@ psyco_conn_commit(connectionObject *self, PyObject *args)
 
     if (!PyArg_ParseTuple(args, "")) return NULL;
 
-    if (conn_commit(self) < 0) {
-        PyErr_SetString(OperationalError,
-                        PQerrorMessage(self->pgconn));
+    if (conn_commit(self) < 0)
         return NULL;
-    }
 
     Py_INCREF(Py_None);
     return Py_None;
@@ -140,11 +137,8 @@ psyco_conn_rollback(connectionObject *self, PyObject *args)
 
     if (!PyArg_ParseTuple(args, "")) return NULL;
 
-    if (conn_rollback(self) < 0) {
-        PyErr_SetString(OperationalError,
-                        PQerrorMessage(self->pgconn));
+    if (conn_rollback(self) < 0)
         return NULL;
-    }
 
     Py_INCREF(Py_None);
     return Py_None;
@@ -192,7 +186,8 @@ psyco_conn_set_isolation_level(connectionObject *self, PyObject *args)
 static PyObject *
 psyco_conn_set_client_encoding(connectionObject *self, PyObject *args)
 {
-    char *buffer, *enc = NULL;
+    const char *enc = NULL;
+    char *buffer;
     size_t i, j;
 
     EXC_IF_CONN_CLOSED(self);
@@ -200,12 +195,12 @@ psyco_conn_set_client_encoding(connectionObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "s", &enc)) return NULL;
 
     /* convert to upper case and remove '-' and '_' from string */
-    buffer = PyMem_Malloc(strlen(enc));
+    buffer = PyMem_Malloc(strlen(enc)+1);
     for (i=j=0 ; i < strlen(enc) ; i++) {
         if (enc[i] == '_' || enc[i] == '-')
-        continue;
-    else
-        buffer[j++] = toupper(enc[i]);
+            continue;
+        else
+            buffer[j++] = toupper(enc[i]);
     }
     buffer[j] = '\0';
 
@@ -237,6 +232,14 @@ psyco_conn_get_transaction_status(connectionObject *self, PyObject *args)
 
 #endif
 
+static PyObject *
+psyco_conn_get_exception(PyObject *self, void *closure)
+{
+    PyObject *exception = *(PyObject **)closure;
+
+    Py_INCREF(exception);
+    return exception;
+}
 
 /** the connection object **/
 
@@ -266,32 +269,6 @@ static struct PyMethodDef connectionObject_methods[] = {
 /* object member list */
 
 static struct PyMemberDef connectionObject_members[] = {
-    /* DBAPI-2.0 extensions (exception objects) */
-    {"Error", T_OBJECT,
-        offsetof(connectionObject, exc_Error), RO, Error_doc},
-    {"Warning",
-        T_OBJECT, offsetof(connectionObject, exc_Warning), RO, Warning_doc},
-    {"InterfaceError", T_OBJECT,
-        offsetof(connectionObject, exc_InterfaceError), RO,
-        InterfaceError_doc},
-    {"DatabaseError", T_OBJECT,
-        offsetof(connectionObject, exc_DatabaseError), RO, DatabaseError_doc},
-    {"InternalError", T_OBJECT,
-        offsetof(connectionObject, exc_InternalError), RO, InternalError_doc},
-    {"OperationalError", T_OBJECT,
-        offsetof(connectionObject, exc_OperationalError), RO,
-        OperationalError_doc},
-    {"ProgrammingError", T_OBJECT,
-        offsetof(connectionObject, exc_ProgrammingError), RO,
-        ProgrammingError_doc},
-    {"IntegrityError", T_OBJECT,
-        offsetof(connectionObject, exc_IntegrityError), RO,
-        IntegrityError_doc},
-    {"DataError", T_OBJECT,
-        offsetof(connectionObject, exc_DataError), RO, DataError_doc},
-    {"NotSupportedError", T_OBJECT,
-        offsetof(connectionObject, exc_NotSupportedError), RO,
-        NotSupportedError_doc},
 #ifdef PSYCOPG_EXTENSIONS
     {"closed", T_LONG, offsetof(connectionObject, closed), RO,
         "True if the connection is closed."},
@@ -315,10 +292,29 @@ static struct PyMemberDef connectionObject_members[] = {
     {NULL}
 };
 
+#define EXCEPTION_GETTER(exc) \
+    { #exc, psyco_conn_get_exception, NULL, exc ## _doc, &exc }
+
+static struct PyGetSetDef connectionObject_getsets[] = {
+    /* DBAPI-2.0 extensions (exception objects) */
+    EXCEPTION_GETTER(Error),
+    EXCEPTION_GETTER(Warning),
+    EXCEPTION_GETTER(InterfaceError),
+    EXCEPTION_GETTER(DatabaseError),
+    EXCEPTION_GETTER(InternalError),
+    EXCEPTION_GETTER(OperationalError),
+    EXCEPTION_GETTER(ProgrammingError),
+    EXCEPTION_GETTER(IntegrityError),
+    EXCEPTION_GETTER(DataError),
+    EXCEPTION_GETTER(NotSupportedError),
+    {NULL}
+};
+#undef EXCEPTION_GETTER
+
 /* initialization and finalization methods */
 
 static int
-connection_setup(connectionObject *self, char *dsn)
+connection_setup(connectionObject *self, const char *dsn)
 {
     char *pos;
     int res;
@@ -372,7 +368,7 @@ connection_dealloc(PyObject* obj)
     if (self->closed == 0) conn_close(self);
 
     if (self->dsn) free(self->dsn);
-    if (self->encoding) PyMem_Free(self->encoding);
+    if (self->encoding) free(self->encoding);
     if (self->critical) free(self->critical);
 
     Py_XDECREF(self->notice_list);
@@ -394,7 +390,7 @@ connection_dealloc(PyObject* obj)
 static int
 connection_init(PyObject *obj, PyObject *args, PyObject *kwds)
 {
-    char *dsn;
+    const char *dsn;
 
     if (!PyArg_ParseTuple(args, "s", &dsn))
         return -1;
@@ -471,7 +467,7 @@ PyTypeObject connectionType = {
 
     connectionObject_methods, /*tp_methods*/
     connectionObject_members, /*tp_members*/
-    0,          /*tp_getset*/
+    connectionObject_getsets, /*tp_getset*/
     0,          /*tp_base*/
     0,          /*tp_dict*/
 
