@@ -308,15 +308,18 @@ _psyco_curs_execute(cursorObject *self,
     int res = 0;
     PyObject *fquery, *cvt = NULL;
 
+    Py_BEGIN_ALLOW_THREADS;
     pthread_mutex_lock(&(self->conn->lock));
     if (self->conn->async_cursor != NULL
         && self->conn->async_cursor != (PyObject*)self) {
         pthread_mutex_unlock(&(self->conn->lock));
+        Py_BLOCK_THREADS;
         psyco_set_error(ProgrammingError, (PyObject*)self,
                          "asynchronous query already in execution", NULL, NULL);
         return 0;
     }
     pthread_mutex_unlock(&(self->conn->lock));
+    Py_END_ALLOW_THREADS;
 
     operation = _psyco_curs_validate_sql_basic(self, operation);
 
@@ -368,7 +371,7 @@ _psyco_curs_execute(cursorObject *self,
                 if (PyObject_HasAttrString(arg, "args")) {
                     PyObject *args = PyObject_GetAttrString(arg, "args");
                     PyObject *str = PySequence_GetItem(args, 0);
-                    char *s = PyString_AS_STRING(str);
+                    const char *s = PyString_AS_STRING(str);
 
                     Dprintf("psyco_curs_execute:     -> %s", s);
 
@@ -525,6 +528,7 @@ psyco_curs_executemany(cursorObject *self, PyObject *args, PyObject *kwargs)
         }
     }
     Py_XDECREF(iter);
+    self->rowcount = -1;
 
     Py_INCREF(Py_None);
     return Py_None;
@@ -582,7 +586,7 @@ psyco_curs_mogrify(cursorObject *self, PyObject *args, PyObject *kwargs)
                 if (PyObject_HasAttrString(arg, "args")) {
                     PyObject *args = PyObject_GetAttrString(arg, "args");
                     PyObject *str = PySequence_GetItem(args, 0);
-                    char *s = PyString_AS_STRING(str);
+                    const char *s = PyString_AS_STRING(str);
 
                     Dprintf("psyco_curs_execute:     -> %s", s);
 
@@ -640,15 +644,18 @@ _psyco_curs_prefetch(cursorObject *self)
 
     /* check if the fetching cursor is the one that did the asynchronous query
        and raise an exception if not */
+    Py_BEGIN_ALLOW_THREADS;
     pthread_mutex_lock(&(self->conn->lock));
     if (self->conn->async_cursor != NULL
         && self->conn->async_cursor != (PyObject*)self) {
         pthread_mutex_unlock(&(self->conn->lock));
+        Py_BLOCK_THREADS;
         psyco_set_error(ProgrammingError, (PyObject*)self,
                          "asynchronous fetch by wrong cursor", NULL, NULL);
         return -2;
     }
     pthread_mutex_unlock(&(self->conn->lock));
+    Py_END_ALLOW_THREADS;
 
     if (self->pgres == NULL || self->needsfetch) {
         self->needsfetch = 0;
@@ -668,7 +675,7 @@ _psyco_curs_buildrow_fill(cursorObject *self, PyObject *res,
                           int row, int n, int istuple)
 {
     int i, len;
-    unsigned char *str;
+    const char *str;
     PyObject *val;
 
     for (i=0; i < n; i++) {
@@ -677,14 +684,14 @@ _psyco_curs_buildrow_fill(cursorObject *self, PyObject *res,
             len = 0;
         }
         else {
-            str = (unsigned char*)PQgetvalue(self->pgres, row, i);
+            str = PQgetvalue(self->pgres, row, i);
             len = PQgetlength(self->pgres, row, i);
         }
 
         Dprintf("_psyco_curs_buildrow: row %ld, element %d, len %d",
                 self->row, i, len);
 
-        val = typecast_cast(PyTuple_GET_ITEM(self->casts, i), (char*)str, len,
+        val = typecast_cast(PyTuple_GET_ITEM(self->casts, i), str, len,
                             (PyObject*)self);
 
         if (val) {
@@ -734,7 +741,7 @@ _psyco_curs_buildrow_with_factory(cursorObject *self, int row)
     return _psyco_curs_buildrow_fill(self, res, row, n, 0);
 }
 
-PyObject *
+static PyObject *
 psyco_curs_fetchone(cursorObject *self, PyObject *args)
 {
     PyObject *res;
@@ -788,7 +795,7 @@ psyco_curs_fetchone(cursorObject *self, PyObject *args)
 "of tuples (by default) or using the sequence factory previously set in\n" \
 "the `row_factory` attribute. Return `None` when no more data is available.\n"
 
-PyObject *
+static PyObject *
 psyco_curs_fetchmany(cursorObject *self, PyObject *args, PyObject *kwords)
 {
     int i;
@@ -863,7 +870,7 @@ psyco_curs_fetchmany(cursorObject *self, PyObject *args, PyObject *kwords)
 "the sequence factory previously set in the `row_factory` attribute.\n" \
 "Return `None` when no more data is available.\n"
 
-PyObject *
+static PyObject *
 psyco_curs_fetchall(cursorObject *self, PyObject *args)
 {
     int i, size;
@@ -928,10 +935,11 @@ psyco_curs_fetchall(cursorObject *self, PyObject *args)
 static PyObject *
 psyco_curs_callproc(cursorObject *self, PyObject *args, PyObject *kwargs)
 {
-    char *procname = NULL, *sql = NULL;
+    const char *procname = NULL;
+    char *sql = NULL;
     long int async = 0;
     Py_ssize_t procname_len, i, nparameters = 0, sl = 0;
-    PyObject *parameters = NULL;
+    PyObject *parameters = Py_None;
     PyObject *operation = NULL;
     PyObject *res = NULL;
 
@@ -948,9 +956,9 @@ psyco_curs_callproc(cursorObject *self, PyObject *args, PyObject *kwargs)
         return NULL;
     }
 
-    if(parameters && parameters != Py_None) {
+    if(parameters != Py_None) {
         nparameters = PyObject_Length(parameters);
-    if (nparameters < 0) nparameters = 0;
+        if (nparameters < 0) nparameters = 0;
     }
 
     /* allocate some memory, build the SQL and create a PyString from it */
@@ -969,8 +977,8 @@ psyco_curs_callproc(cursorObject *self, PyObject *args, PyObject *kwargs)
     PyMem_Free((void*)sql);
 
     if (_psyco_curs_execute(self, operation, parameters, async)) {
-        Py_INCREF(Py_None);
-        res = Py_None;
+        Py_INCREF(parameters);
+        res = parameters;
     }
 
     Py_DECREF(operation);
@@ -1048,7 +1056,7 @@ static PyObject *
 psyco_curs_scroll(cursorObject *self, PyObject *args, PyObject *kwargs)
 {
     int value, newpos;
-    char *mode = "relative";
+    const char *mode = "relative";
 
     static char *kwlist[] = {"value", "mode", NULL};
 
@@ -1179,8 +1187,8 @@ static PyObject *
 psyco_curs_copy_from(cursorObject *self, PyObject *args, PyObject *kwargs)
 {
     char query[DEFAULT_COPYBUFF];
-    char *table_name;
-    char *sep = "\t", *null = NULL;
+    const char *table_name;
+    const char *sep = "\t", *null = NULL;
     Py_ssize_t bufsize = DEFAULT_COPYBUFF;
     PyObject *file, *columns = NULL, *res = NULL;
     char columnlist[DEFAULT_COPYBUFF];
@@ -1253,8 +1261,8 @@ psyco_curs_copy_to(cursorObject *self, PyObject *args, PyObject *kwargs)
 {
     char query[DEFAULT_COPYBUFF];
     char columnlist[DEFAULT_COPYBUFF];
-    char *table_name;
-    char *sep = "\t", *null = NULL;
+    const char *table_name;
+    const char *sep = "\t", *null = NULL;
     PyObject *file, *columns = NULL, *res = NULL;
 
     static char *kwlist[] = {"file", "table", "sep", "null", "columns", NULL};
@@ -1381,12 +1389,12 @@ psyco_curs_fileno(cursorObject *self, PyObject *args)
 
     /* note how we call PQflush() to make sure the user will use
        select() in the safe way! */
-    pthread_mutex_lock(&(self->conn->lock));
     Py_BEGIN_ALLOW_THREADS;
+    pthread_mutex_lock(&(self->conn->lock));
     PQflush(self->conn->pgconn);
     socket = (long int)PQsocket(self->conn->pgconn);
-    Py_END_ALLOW_THREADS;
     pthread_mutex_unlock(&(self->conn->lock));
+    Py_END_ALLOW_THREADS;
 
     return PyInt_FromLong(socket);
 }
@@ -1412,16 +1420,34 @@ psyco_curs_isready(cursorObject *self, PyObject *args)
     }
     else {
         IFCLEARPGRES(self->pgres);
+        Py_BEGIN_ALLOW_THREADS;
         pthread_mutex_lock(&(self->conn->lock));
         self->pgres = PQgetResult(self->conn->pgconn);
         self->conn->async_cursor = NULL;
         pthread_mutex_unlock(&(self->conn->lock));
+        Py_END_ALLOW_THREADS;
         self->needsfetch = 1;
         Py_INCREF(Py_True);
         return Py_True;
     }
 }
 
+/* extension: closed - return true if cursor is closed*/
+
+#define psyco_curs_closed_doc \
+"True if cursor is closed, False if cursor is open"
+
+static PyObject *
+psyco_curs_get_closed(cursorObject *self, void *closure)
+{
+    PyObject *closed;
+
+    closed = (self->closed || (self->conn && self->conn->closed)) ?
+        Py_True : Py_False;
+    Py_INCREF(closed);
+    return closed;
+}
+
 #endif
 
 
@@ -1533,10 +1559,19 @@ static struct PyMemberDef cursorObject_members[] = {
     {NULL}
 };
 
+/* object calculated member list */
+static struct PyGetSetDef cursorObject_getsets[] = {
+#ifdef PSYCOPG_EXTENSIONS
+    { "closed", (getter)psyco_curs_get_closed, NULL,
+      psyco_curs_closed_doc, NULL },
+#endif
+    {NULL}
+};
+
 /* initialization and finalization methods */
 
 static int
-cursor_setup(cursorObject *self, connectionObject *conn, char *name)
+cursor_setup(cursorObject *self, connectionObject *conn, const char *name)
 {
     Dprintf("cursor_setup: init cursor object at %p", self);
     Dprintf("cursor_setup:     parameters: name = %s, conn = %p", name, conn);
@@ -1621,7 +1656,7 @@ cursor_dealloc(PyObject* obj)
 static int
 cursor_init(PyObject *obj, PyObject *args, PyObject *kwds)
 {
-    char *name = NULL;
+    const char *name = NULL;
     PyObject *conn;
 
     if (!PyArg_ParseTuple(args, "O|s", &conn, &name))
@@ -1694,7 +1729,7 @@ PyTypeObject cursorType = {
 
     cursorObject_methods, /*tp_methods*/
     cursorObject_members, /*tp_members*/
-    0,          /*tp_getset*/
+    cursorObject_getsets, /*tp_getset*/
     0,          /*tp_base*/
     0,          /*tp_dict*/
 
