@@ -40,30 +40,132 @@
 #include "psycopg/pgtypes.h"
 #include "psycopg/pgversion.h"
 
+
+/* Strip off the severity from a Postgres error message. */
+static const char *
+strip_severity(const char *msg)
+{
+    if (!msg)
+        return NULL;
+
+    if (strlen(msg) > 8 && (!strncmp(msg, "ERROR:  ", 8) ||
+                            !strncmp(msg, "FATAL:  ", 8) ||
+                            !strncmp(msg, "PANIC:  ", 8)))
+        return &msg[8];
+    else
+        return msg;
+}
+
+/* Returns the Python exception corresponding to an SQLSTATE error
+   code.  A list of error codes can be found at:
+
+   http://www.postgresql.org/docs/current/static/errcodes-appendix.html */
+static PyObject *
+exception_from_sqlstate(const char *sqlstate)
+{
+    switch (sqlstate[0]) {
+    case '0':
+        switch (sqlstate[1]) {
+        case 'A': /* Class 0A - Feature Not Supported */
+            return NotSupportedError;
+        }
+        break;
+    case '2':
+        switch (sqlstate[1]) {
+        case '1': /* Class 21 - Cardinality Violation */
+            return ProgrammingError;
+        case '2': /* Class 22 - Data Exception */
+            return DataError;
+        case '3': /* Class 23 - Integrity Constraint Violation */
+            return IntegrityError;
+        case '4': /* Class 24 - Invalid Cursor State */
+        case '5': /* Class 25 - Invalid Transaction State */
+            return InternalError;
+        case '6': /* Class 26 - Invalid SQL Statement Name */
+        case '7': /* Class 27 - Triggered Data Change Violation */
+        case '8': /* Class 28 - Invalid Authorization Specification */
+            return OperationalError;
+        case 'B': /* Class 2B - Dependent Privilege Descriptors Still Exist */
+        case 'D': /* Class 2D - Invalid Transaction Termination */
+        case 'F': /* Class 2F - SQL Routine Exception */
+            return InternalError;
+        }
+        break;
+    case '3':
+        switch (sqlstate[1]) {
+        case '4': /* Class 34 - Invalid Cursor Name */
+            return OperationalError;
+        case '8': /* Class 38 - External Routine Exception */
+        case '9': /* Class 39 - External Routine Invocation Exception */
+        case 'B': /* Class 3B - Savepoint Exception */
+            return InternalError;
+        case 'D': /* Class 3D - Invalid Catalog Name */
+        case 'F': /* Class 3F - Invalid Schema Name */
+            return ProgrammingError;
+        }
+        break;
+    case '4':
+        switch (sqlstate[1]) {
+        case '0': /* Class 40 - Transaction Rollback */
+#ifdef PSYCOPG_EXTENSIONS
+            return TransactionRollbackError;
+#else
+            return OperationalError;
+#endif
+        case '2': /* Class 42 - Syntax Error or Access Rule Violation */
+        case '4': /* Class 44 - WITH CHECK OPTION Violation */
+            return ProgrammingError;
+        }
+        break;
+    case '5':
+        /* Class 53 - Insufficient Resources
+           Class 54 - Program Limit Exceeded
+           Class 55 - Object Not In Prerequisite State
+           Class 57 - Operator Intervention
+           Class 58 - System Error (errors external to PostgreSQL itself) */
+#ifdef PSYCOPG_EXTENSIONS
+        if (!strcmp(sqlstate, "57014"))
+            return QueryCanceledError;
+        else
+#endif
+            return OperationalError;
+    case 'F': /* Class F0 - Configuration File Error */
+        return InternalError;
+    case 'P': /* Class P0 - PL/pgSQL Error */
+        return InternalError;
+    case 'X': /* Class XX - Internal Error */
+        return InternalError;
+    }
+    /* return DatabaseError as a fallback */
+    return DatabaseError;
+}
+
 /* pq_raise - raise a python exception of the right kind
 
    This function should be called while holding the GIL. */
 
-void
-pq_raise(connectionObject *conn, cursorObject *curs, PyObject *exc, char *msg)
+static void
+pq_raise(connectionObject *conn, cursorObject *curs, PGresult *pgres)
 {
     PyObject *pgc = (PyObject*)curs;
-
-    char *err = NULL;
-    char *err2 = NULL;
-    char *code = NULL;
-    char *buf = NULL;
+    PyObject *exc = NULL;
+    const char *err = NULL;
+    const char *err2 = NULL;
+    const char *code = NULL;
 
     if ((conn == NULL && curs == NULL) || (curs != NULL && conn == NULL)) {
         PyErr_SetString(Error, "psycopg went psycotic and raised a null error");
         return;
     }
 
-    if (curs && curs->pgres) {
-        err = PQresultErrorMessage(curs->pgres);
+    if (pgres == NULL && curs != NULL)
+        pgres = curs->pgres;
+
+    if (pgres) {
+        err = PQresultErrorMessage(pgres);
 #ifdef HAVE_PQPROTOCOL3
         if (err != NULL && conn->protocol == 3) {
-            code = PQresultErrorField(curs->pgres, PG_DIAG_SQLSTATE);
+            code = PQresultErrorField(pgres, PG_DIAG_SQLSTATE);
         }
 #endif
     }
@@ -78,21 +180,10 @@ pq_raise(connectionObject *conn, cursorObject *curs, PyObject *exc, char *msg)
         return;
     }
 
-    /* if exc is NULL, analyze the message and try to deduce the right
-       exception kind (only if we have a pgres, obviously) */
-    if (exc == NULL) {
-        if (curs && curs->pgres) {
-            if (conn->protocol == 3) {
-#ifdef HAVE_PQPROTOCOL3
-                char *pgstate =
-                    PQresultErrorField(curs->pgres, PG_DIAG_SQLSTATE);
-                if (pgstate != NULL && !strncmp(pgstate, "23", 2))
-                    exc = IntegrityError;
-                else
-                    exc = ProgrammingError;
-#endif
-            }
-        }
+    /* Analyze the message and try to deduce the right exception kind
+       (only if we got the SQLSTATE from the pgres, obviously) */
+    if (code != NULL) {
+        exc = exception_from_sqlstate(code);
     }
 
     /* if exc is still NULL psycopg was not built with HAVE_PQPROTOCOL3 or the
@@ -103,34 +194,21 @@ pq_raise(connectionObject *conn, cursorObject *curs, PyObject *exc, char *msg)
             || !strncmp(err, "ERROR:  ExecAppend: Fail to add null", 36)
             || strstr(err, "referential integrity violation"))
             exc = IntegrityError;
+        else if (strstr(err, "could not serialize") ||
+                 strstr(err, "deadlock detected"))
+#ifdef PSYCOPG_EXTENSIONS
+            exc = TransactionRollbackError;
+#else
+            exc = OperationalError;
+#endif
         else
             exc = ProgrammingError;
     }
 
     /* try to remove the initial "ERROR: " part from the postgresql error */
-    if (err && strlen(err) > 8) err2 = &(err[8]);
-    else err2 = err;
+    err2 = strip_severity(err);
 
-    /* if msg is not NULL, add it to the error message, after a '\n' */
-    if (msg && code) {
-        size_t len = strlen(code) + strlen(err) + strlen(msg) + 5;
-        if ((buf = PyMem_Malloc(len))) {
-            snprintf(buf, len, "[%s] %s\n%s", code, err2, msg);
-            psyco_set_error(exc, pgc, buf, err, code);
-        }
-    }
-    else if (msg) {
-        size_t len = strlen(err) + strlen(msg) + 2;
-        if ((buf = PyMem_Malloc(len))) {
-            snprintf(buf, len, "%s\n%s", err2, msg);
-            psyco_set_error(exc, pgc, buf, err, code);
-        }
-    }
-    else {
-        psyco_set_error(exc, pgc, err2, err, code);
-    }
-
-    if (buf != NULL) PyMem_Free(buf);
+    psyco_set_error(exc, pgc, err2, err, code);
 }
 
 /* pq_set_critical, pq_resolve_critical - manage critical errors
@@ -148,11 +226,28 @@ pq_set_critical(connectionObject *conn, const char *msg)
     if (msg == NULL)
         msg = PQerrorMessage(conn->pgconn);
     if (conn->critical) free(conn->critical);
+    Dprintf("pq_set_critical: setting %s", msg);
     if (msg && msg[0] != '\0') conn->critical = strdup(msg);
     else conn->critical = NULL;
 }
 
-PyObject *
+static void
+pq_clear_critical(connectionObject *conn)
+{
+    /* sometimes we know that the notice analizer set a critical that
+       was not really as such (like when raising an error for a delayed
+       contraint violation. it would be better to analyze the notice
+       or avoid the set-error-on-notice stuff at all but given that we
+       can't, some functions at least clear the critical status after
+       operations they know would result in a wrong critical to be set */
+    Dprintf("pq_clear_critical: clearing %s", conn->critical);
+    if (conn->critical) {
+        free(conn->critical);
+        conn->critical = NULL;
+    }
+}
+
+static PyObject *
 pq_resolve_critical(connectionObject *conn, int close)
 {
     Dprintf("pq_resolve_critical: resolving %s", conn->critical);
@@ -167,6 +262,9 @@ pq_resolve_critical(connectionObject *conn, int close)
 
         /* we don't want to destroy this connection but just close it */
         if (close == 1) conn_close(conn);
+    
+        /* remember to clear the critical! */
+        pq_clear_critical(conn);    
     }
     return NULL;
 }
@@ -179,7 +277,7 @@ pq_resolve_critical(connectionObject *conn, int close)
 
    this function does not call any Py_*_ALLOW_THREADS macros */
 
-void
+static void
 pq_clear_async(connectionObject *conn)
 {
     PGresult *pgres;
@@ -191,63 +289,122 @@ pq_clear_async(connectionObject *conn)
     } while (pgres != NULL);
 }
 
-/* pq_begin - send a BEGIN WORK, if necessary
+/* pg_execute_command_locked - execute a no-result query on a locked connection.
 
-   this function does not call any Py_*_ALLOW_THREADS macros */
+   This function should only be called on a locked connection without
+   holding the global interpreter lock.
 
+   On error, -1 is returned, and the pgres argument will hold the
+   relevant result structure.
+ */
 int
-pq_begin(connectionObject *conn)
+pq_execute_command_locked(connectionObject *conn, const char *query,
+                          PGresult **pgres, char **error)
+{
+    int pgstatus, retvalue = -1;
+
+    Dprintf("pq_execute_command_locked: pgconn = %p, query = %s",
+            conn->pgconn, query);
+    *error = NULL;
+    *pgres = PQexec(conn->pgconn, query);
+    if (*pgres == NULL) {
+        const char *msg;
+
+        Dprintf("pq_execute_command_locked: PQexec returned NULL");
+        msg = PQerrorMessage(conn->pgconn);
+        if (msg)
+            *error = strdup(msg);
+        goto cleanup;
+    }
+
+    pgstatus = PQresultStatus(*pgres);
+    if (pgstatus != PGRES_COMMAND_OK ) {
+        Dprintf("pq_execute_command_locked: result was not COMMAND_OK (%d)",
+                pgstatus);
+        goto cleanup;
+    }
+
+    retvalue = 0;
+    IFCLEARPGRES(*pgres);
+
+ cleanup:
+    return retvalue;
+}
+
+/* pq_complete_error: handle an error from pq_execute_command_locked()
+
+   If pq_execute_command_locked() returns -1, this function should be
+   called to convert the result to a Python exception.
+
+   This function should be called while holding the global interpreter
+   lock.
+ */
+void
+pq_complete_error(connectionObject *conn, PGresult **pgres, char **error)
+{
+    Dprintf("pq_complete_error: pgconn = %p, pgres = %p, error = %s",
+            conn->pgconn, *pgres, *error ? *error : "(null)");
+    if (*pgres != NULL)
+        pq_raise(conn, NULL, *pgres);
+    else if (*error != NULL) {
+        PyErr_SetString(OperationalError, *error);
+        free(*error);
+    } else {
+        PyErr_SetString(OperationalError, "unknown error");
+    }
+    IFCLEARPGRES(*pgres);
+    if (*error) {
+        free(*error);
+        *error = NULL;
+    }
+}
+
+
+/* pq_begin_locked - begin a transaction, if necessary
+
+   This function should only be called on a locked connection without
+   holding the global interpreter lock.
+
+   On error, -1 is returned, and the pgres argument will hold the
+   relevant result structure.
+ */
+static int
+pq_begin_locked(connectionObject *conn, PGresult **pgres, char **error)
 {
     const char *query[] = {
         NULL,
         "BEGIN; SET TRANSACTION ISOLATION LEVEL READ COMMITTED",
         "BEGIN; SET TRANSACTION ISOLATION LEVEL SERIALIZABLE"};
+    int result;
 
-    int pgstatus, retvalue = -1;
-    PGresult *pgres = NULL;
-
-    Dprintf("pq_begin: pgconn = %p, isolevel = %ld, status = %d",
+    Dprintf("pq_begin_locked: pgconn = %p, isolevel = %ld, status = %d",
             conn->pgconn, conn->isolation_level, conn->status);
 
     if (conn->isolation_level == 0 || conn->status != CONN_STATUS_READY) {
-        Dprintf("pq_begin: transaction in progress");
+        Dprintf("pq_begin_locked: transaction in progress");
         return 0;
     }
 
     pq_clear_async(conn);
-    pgres = PQexec(conn->pgconn, query[conn->isolation_level]);
-    if (pgres == NULL) {
-        Dprintf("pq_begin: PQexec() failed");
-        pq_set_critical(conn, NULL);
-        goto cleanup;
-    }
+    result = pq_execute_command_locked(conn, query[conn->isolation_level],
+                                       pgres, error);
+    if (result == 0)
+        conn->status = CONN_STATUS_BEGIN;
 
-    pgstatus = PQresultStatus(pgres);
-    if (pgstatus != PGRES_COMMAND_OK ) {
-        Dprintf("pq_begin: result is NOT OK");
-        pq_set_critical(conn, NULL);
-        goto cleanup;
-    }
-    Dprintf("pq_begin: issued '%s' command", query[conn->isolation_level]);
-
-    retvalue = 0;
-    conn->status = CONN_STATUS_BEGIN;
-
- cleanup:
-    IFCLEARPGRES(pgres);
-    return retvalue;
+    return result;
 }
 
 /* pq_commit - send an END, if necessary
 
-   this function does not call any Py_*_ALLOW_THREADS macros */
+   This function should be called while holding the global interpreter
+   lock. */
 
 int
 pq_commit(connectionObject *conn)
 {
-    const char *query = "END";
-    int pgstatus, retvalue = -1;
+    int retvalue = -1;
     PGresult *pgres = NULL;
+    char *error = NULL;
 
     Dprintf("pq_commit: pgconn = %p, isolevel = %ld, status = %d",
             conn->pgconn, conn->isolation_level, conn->status);
@@ -257,40 +414,57 @@ pq_commit(connectionObject *conn)
         return 0;
     }
 
+    Py_BEGIN_ALLOW_THREADS;
+    pthread_mutex_lock(&conn->lock);
+
     pq_clear_async(conn);
-    pgres = PQexec(conn->pgconn, query);
-    if (pgres == NULL) {
-        Dprintf("pq_commit: PQexec() failed");
-        pq_set_critical(conn, NULL);
-        goto cleanup;
-    }
+    retvalue = pq_execute_command_locked(conn, "COMMIT", &pgres, &error);
 
-    pgstatus = PQresultStatus(pgres);
-    if (pgstatus != PGRES_COMMAND_OK ) {
-        Dprintf("pq_commit: result is NOT OK");
-        pq_set_critical(conn, NULL);
-        goto cleanup;
-    }
-    Dprintf("pq_commit: issued '%s' command", query);
+    pthread_mutex_unlock(&conn->lock);
+    Py_END_ALLOW_THREADS;
 
-    retvalue = 0;
+    if (retvalue < 0)
+        pq_complete_error(conn, &pgres, &error);
+
+    /* Even if an error occurred, the connection will be rolled back,
+       so we unconditionally set the connection status here. */
     conn->status = CONN_STATUS_READY;
 
- cleanup:
-    IFCLEARPGRES(pgres);
+    return retvalue;
+}
+
+int
+pq_abort_locked(connectionObject *conn, PGresult **pgres, char **error)
+{
+    int retvalue = -1;
+
+    Dprintf("pq_abort_locked: pgconn = %p, isolevel = %ld, status = %d",
+            conn->pgconn, conn->isolation_level, conn->status);
+
+    if (conn->isolation_level == 0 || conn->status != CONN_STATUS_BEGIN) {
+        Dprintf("pq_abort_locked: no transaction to abort");
+        return 0;
+    }
+
+    pq_clear_async(conn);
+    retvalue = pq_execute_command_locked(conn, "ROLLBACK", pgres, error);
+    if (retvalue == 0)
+        conn->status = CONN_STATUS_READY;
+
     return retvalue;
 }
 
 /* pq_abort - send an ABORT, if necessary
 
-   this function does not call any Py_*_ALLOW_THREADS macros */
+   This function should be called while holding the global interpreter
+   lock. */
 
 int
 pq_abort(connectionObject *conn)
 {
-    const char *query = "ABORT";
-    int pgstatus, retvalue = -1;
+    int retvalue = -1;
     PGresult *pgres = NULL;
+    char *error = NULL;
 
     Dprintf("pq_abort: pgconn = %p, isolevel = %ld, status = %d",
             conn->pgconn, conn->isolation_level, conn->status);
@@ -300,27 +474,17 @@ pq_abort(connectionObject *conn)
         return 0;
     }
 
-    pq_clear_async(conn);
-    pgres = PQexec(conn->pgconn, query);
-    if (pgres == NULL) {
-        Dprintf("pq_abort: PQexec() failed");
-        pq_set_critical(conn, NULL);
-        goto cleanup;
-    }
+    Py_BEGIN_ALLOW_THREADS;
+    pthread_mutex_lock(&conn->lock);
 
-    pgstatus = PQresultStatus(pgres);
-    if (pgstatus != PGRES_COMMAND_OK ) {
-        Dprintf("pq_abort: result is NOT OK");
-        pq_set_critical(conn, NULL);
-        goto cleanup;
-    }
-    Dprintf("pq_abort: issued '%s' command", query);
+    retvalue = pq_abort_locked(conn, &pgres, &error);
 
-    retvalue = 0;
-    conn->status = CONN_STATUS_READY;
+    pthread_mutex_unlock(&conn->lock);
+    Py_END_ALLOW_THREADS;
+
+    if (retvalue < 0)
+        pq_complete_error(conn, &pgres, &error);
 
- cleanup:
-    IFCLEARPGRES(pgres);
     return retvalue;
 }
 
@@ -336,6 +500,7 @@ pq_abort(connectionObject *conn)
 int
 pq_is_busy(connectionObject *conn)
 {
+    int res;
     PGnotify *pgn;
 
     Dprintf("pq_is_busy: consuming input");
@@ -351,8 +516,6 @@ pq_is_busy(connectionObject *conn)
         return -1;
     }
 
-    pthread_mutex_unlock(&(conn->lock));
-    Py_END_ALLOW_THREADS;
 
     /* now check for notifies */
     while ((pgn = PQnotifies(conn->pgconn)) != NULL) {
@@ -361,14 +524,21 @@ pq_is_busy(connectionObject *conn)
         Dprintf("curs_is_busy: got NOTIFY from pid %d, msg = %s",
                 (int) pgn->be_pid, pgn->relname);
 
+        Py_BLOCK_THREADS;
         notify = PyTuple_New(2);
         PyTuple_SET_ITEM(notify, 0, PyInt_FromLong((long)pgn->be_pid));
         PyTuple_SET_ITEM(notify, 1, PyString_FromString(pgn->relname));
         PyList_Append(conn->notifies, notify);
+        Py_UNBLOCK_THREADS;
         free(pgn);
     }
 
-    return PQisBusy(conn->pgconn);
+    res = PQisBusy(conn->pgconn);
+    
+    pthread_mutex_unlock(&(conn->lock));
+    Py_END_ALLOW_THREADS;
+
+    return res;
 }
 
 /* pq_execute - execute a query, possibly asyncronously
@@ -379,6 +549,9 @@ pq_is_busy(connectionObject *conn)
 int
 pq_execute(cursorObject *curs, const char *query, int async)
 {
+    PGresult *pgres = NULL;
+    char *error = NULL;
+
     /* if the status of the connection is critical raise an exception and
        definitely close the connection */
     if (curs->conn->critical) {
@@ -397,11 +570,10 @@ pq_execute(cursorObject *curs, const char *query, int async)
     Py_BEGIN_ALLOW_THREADS;
     pthread_mutex_lock(&(curs->conn->lock));
 
-    if (pq_begin(curs->conn) < 0) {
+    if (pq_begin_locked(curs->conn, &pgres, &error) < 0) {
         pthread_mutex_unlock(&(curs->conn->lock));
         Py_BLOCK_THREADS;
-        PyErr_SetString(OperationalError,
-                        PQerrorMessage(curs->conn->pgconn));
+        pq_complete_error(curs->conn, &pgres, &error);
         return -1;
     }
 
@@ -412,14 +584,13 @@ pq_execute(cursorObject *curs, const char *query, int async)
         curs->pgres = PQexec(curs->conn->pgconn, query);
 
         /* dont let pgres = NULL go to pq_fetch() */
-        /* if (curs->pgres == NULL) {
+        if (curs->pgres == NULL) {
             pthread_mutex_unlock(&(curs->conn->lock));
             Py_BLOCK_THREADS;
             PyErr_SetString(OperationalError,
                             PQerrorMessage(curs->conn->pgconn));
             return -1;
         }
-        */
     }
 
     else if (async == 1) {
@@ -475,23 +646,32 @@ static void
 _pq_fetch_tuples(cursorObject *curs)
 {
     int i, *dsize = NULL;
+    int pgnfields;
+    int pgbintuples;
 
-    int pgnfields = PQnfields(curs->pgres);
-    int pgbintuples = PQbinaryTuples(curs->pgres);
+    Py_BEGIN_ALLOW_THREADS;
+    pthread_mutex_lock(&(curs->conn->lock));
+    
+    pgnfields = PQnfields(curs->pgres);
+    pgbintuples = PQbinaryTuples(curs->pgres);
 
     curs->notuples = 0;
 
     /* create the tuple for description and typecasting */
+    Py_BLOCK_THREADS;
     Py_XDECREF(curs->description);
-    Py_XDECREF(curs->casts);
+    Py_XDECREF(curs->casts);    
     curs->description = PyTuple_New(pgnfields);
     curs->casts = PyTuple_New(pgnfields);
     curs->columns = pgnfields;
+    Py_UNBLOCK_THREADS;
 
     /* calculate the display size for each column (cpu intensive, can be
        switched off at configuration time) */
 #ifdef PSYCOPG_DISPLAY_SIZE
+    Py_BLOCK_THREADS;
     dsize = (int *)PyMem_Malloc(pgnfields * sizeof(int));
+    Py_UNBLOCK_THREADS;
     if (dsize != NULL) {
         int j, len;
         for (i=0; i < pgnfields; i++) {
@@ -512,10 +692,13 @@ _pq_fetch_tuples(cursorObject *curs)
         int fsize = PQfsize(curs->pgres, i);
         int fmod =  PQfmod(curs->pgres, i);
 
-        PyObject *dtitem = PyTuple_New(7);
-        PyObject *type = PyInt_FromLong(ftype);
+        PyObject *dtitem;
+        PyObject *type;
         PyObject *cast = NULL;
 
+        Py_BLOCK_THREADS;
+
+        dtitem = PyTuple_New(7);
         PyTuple_SET_ITEM(curs->description, i, dtitem);
 
         /* fill the right cast function by accessing three different dictionaries:
@@ -524,6 +707,7 @@ _pq_fetch_tuples(cursorObject *curs)
            - the global dictionary (at module level)
            if we get no defined cast use the default one */
 
+        type = PyInt_FromLong(ftype);
         Dprintf("_pq_fetch_tuples: looking for cast %d:", ftype);
         if (curs->string_types != NULL && curs->string_types != Py_None) {
             cast = PyDict_GetItem(curs->string_types, type);
@@ -600,9 +784,18 @@ _pq_fetch_tuples(cursorObject *curs)
         /* 6/ FIXME: null_ok??? */
         Py_INCREF(Py_None);
         PyTuple_SET_ITEM(dtitem, 6, Py_None);
+    
+        Py_UNBLOCK_THREADS;    
     }
 
-    if (dsize) PyMem_Free(dsize);
+    if (dsize) {
+        Py_BLOCK_THREADS;
+        PyMem_Free(dsize);
+        Py_UNBLOCK_THREADS;
+   }
+   
+    pthread_mutex_unlock(&(curs->conn->lock));
+    Py_END_ALLOW_THREADS;
 }
 
 #ifdef HAVE_PQPROTOCOL3
@@ -656,7 +849,7 @@ _pq_copy_in_v3(cursorObject *curs)
     IFCLEARPGRES(curs->pgres);
     while ((curs->pgres = PQgetResult(curs->conn->pgconn)) != NULL) {
         if (PQresultStatus(curs->pgres) == PGRES_FATAL_ERROR)
-            pq_raise(curs->conn, curs, NULL, NULL);
+            pq_raise(curs->conn, curs, NULL);
         IFCLEARPGRES(curs->pgres);
     }
 
@@ -690,7 +883,7 @@ _pq_copy_in(cursorObject *curs)
     IFCLEARPGRES(curs->pgres);
     while ((curs->pgres = PQgetResult(curs->conn->pgconn)) != NULL) {
         if (PQresultStatus(curs->pgres) == PGRES_FATAL_ERROR)
-            pq_raise(curs->conn, curs, NULL, NULL);
+            pq_raise(curs->conn, curs, NULL);
         IFCLEARPGRES(curs->pgres);
     }
 
@@ -727,7 +920,7 @@ _pq_copy_out_v3(cursorObject *curs)
     }
 
     if (len == -2) {
-        pq_raise(curs->conn, NULL, NULL, NULL);
+        pq_raise(curs->conn, curs, NULL);
         return -1;
     }
 
@@ -735,7 +928,7 @@ _pq_copy_out_v3(cursorObject *curs)
     IFCLEARPGRES(curs->pgres);
     while ((curs->pgres = PQgetResult(curs->conn->pgconn)) != NULL) {
         if (PQresultStatus(curs->pgres) == PGRES_FATAL_ERROR)
-            pq_raise(curs->conn, curs, NULL, NULL);
+            pq_raise(curs->conn, curs, NULL);
         IFCLEARPGRES(curs->pgres);
     }
     return 1;
@@ -786,7 +979,7 @@ _pq_copy_out(cursorObject *curs)
     IFCLEARPGRES(curs->pgres);
     while ((curs->pgres = PQgetResult(curs->conn->pgconn)) != NULL) {
         if (PQresultStatus(curs->pgres) == PGRES_FATAL_ERROR)
-            pq_raise(curs->conn, curs, NULL, NULL);
+            pq_raise(curs->conn, curs, NULL);
         IFCLEARPGRES(curs->pgres);
     }
 
@@ -797,6 +990,7 @@ int
 pq_fetch(cursorObject *curs)
 {
     int pgstatus, ex = -1;
+    const char *rowcount;
 
     /* even if we fail, we remove any information about the previous query */
     curs_reset(curs);
@@ -834,9 +1028,15 @@ pq_fetch(cursorObject *curs)
             Py_END_ALLOW_THREADS;
         }
 
+        Py_BEGIN_ALLOW_THREADS;
+        pthread_mutex_lock(&(curs->conn->lock));
+
         Dprintf("pq_fetch: data is probably ready");
         IFCLEARPGRES(curs->pgres);
         curs->pgres = PQgetResult(curs->conn->pgconn);
+
+        pthread_mutex_unlock(&(curs->conn->lock));
+        Py_END_ALLOW_THREADS;
     }
 
     /* check for PGRES_FATAL_ERROR result */
@@ -867,7 +1067,11 @@ pq_fetch(cursorObject *curs)
 
     case PGRES_COMMAND_OK:
         Dprintf("pq_fetch: command returned OK (no tuples)");
-        curs->rowcount = atoi(PQcmdTuples(curs->pgres));
+        rowcount = PQcmdTuples(curs->pgres);
+        if (!rowcount || !rowcount[0])
+          curs->rowcount = -1;
+        else
+          curs->rowcount = atoi(rowcount);
         curs->lastoid = PQoidValue(curs->pgres);
         CLEARPGRES(curs->pgres);
         ex = 1;
@@ -910,7 +1114,7 @@ pq_fetch(cursorObject *curs)
 
     default:
         Dprintf("pq_fetch: uh-oh, something FAILED");
-        pq_raise(curs->conn, curs, NULL, NULL);
+        pq_raise(curs->conn, curs, NULL);
         IFCLEARPGRES(curs->pgres);
         ex = -1;
         break;
