@@ -25,6 +25,8 @@
 #define PY_SSIZE_T_CLEAN
 #include <Python.h>
 
+#include "psycopg/config.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -77,14 +79,18 @@ extern "C" {
 #define PSYCOPG_API_pointers 2
 
 #ifdef PSYCOPG_MODULE
+
     /** This section is used when compiling psycopgmodule.c & co. **/
-extern psyco_errors_fill_RETURN psyco_errors_fill psyco_errors_fill_PROTO;
-extern psyco_errors_set_RETURN psyco_errors_set psyco_errors_set_PROTO;
+HIDDEN psyco_errors_fill_RETURN psyco_errors_fill psyco_errors_fill_PROTO;
+HIDDEN psyco_errors_set_RETURN psyco_errors_set psyco_errors_set_PROTO;
 
 /* global excpetions */
-extern PyObject *Error, *Warning, *InterfaceError, *DatabaseError,
+extern HIDDEN PyObject *Error, *Warning, *InterfaceError, *DatabaseError,
     *InternalError, *OperationalError, *ProgrammingError,
     *IntegrityError, *DataError, *NotSupportedError;
+#ifdef PSYCOPG_EXTENSIONS
+extern HIDDEN PyObject *QueryCanceledError, *TransactionRollbackError;
+#endif
 
 /* python versions and compatibility stuff */
 #ifndef PyMODINIT_FUNC
@@ -122,7 +128,7 @@ import_psycopg(void)
 #endif
 
 /* postgresql<->python encoding map */
-extern PyObject *psycoEncodings;
+extern HIDDEN PyObject *psycoEncodings;
 
 typedef struct {
     char *pgenc;
@@ -130,11 +136,11 @@ typedef struct {
 } encodingPair;
 
 /* the Decimal type, used by the DECIMAL typecaster */
-extern PyObject *decimalType;
+HIDDEN PyObject *psyco_GetDecimalType(void);
 
 /* some utility functions */
-extern void psyco_set_error(PyObject *exc, PyObject *curs,  char *msg,
-                             char *pgerror, char *pgcode);
+HIDDEN void psyco_set_error(PyObject *exc, PyObject *curs,  const char *msg,
+                            const char *pgerror, const char *pgcode);
 
 /* Exceptions docstrings */
 #define Error_doc \
@@ -167,6 +173,14 @@ extern void psyco_set_error(PyObject *exc, PyObject *curs,  char *msg,
 #define NotSupportedError_doc \
 "A not supported datbase API was called."
 
+#ifdef PSYCOPG_EXTENSIONS
+#define QueryCanceledError_doc \
+"Error related to SQL query cancelation."
+
+#define TransactionRollbackError_doc \
+"Error causing transaction rollback (deadlocks, serialisation failures, etc)."
+#endif
+
 #ifdef __cplusplus
 }
 #endif
