@@ -32,15 +32,15 @@
 
 /* usefull function used by some typecasters */
 
-static char *
-skip_until_space(char *s)
+static const char *
+skip_until_space(const char *s)
 {
     while (*s && *s != ' ') s++;
     return s;
 }
 
-static char *
-skip_until_space2(char *s, Py_ssize_t *len)
+static const char *
+skip_until_space2(const char *s, Py_ssize_t *len)
 {
     while (*len > 0 && *s && *s != ' ') {
         s++; (*len)--;
@@ -49,7 +49,7 @@ skip_until_space2(char *s, Py_ssize_t *len)
 }
 
 static int
-typecast_parse_date(char* s, char** t, Py_ssize_t* len,
+typecast_parse_date(const char* s, const char** t, Py_ssize_t* len,
                      int* year, int* month, int* day)
 {
     int acc = -1, cz = 0;
@@ -79,13 +79,20 @@ typecast_parse_date(char* s, char** t, Py_ssize_t* len,
         *day = acc;
         cz += 1;
     }
+
+    /* Is this a BC date?  If so, adjust the year value.  Note that
+     * mx.DateTime numbers BC dates from zero rather than one.  The
+     * Python datetime module does not support BC dates at all. */
+    if (*len >= 2 && s[*len-2] == 'B' && s[*len-1] == 'C')
+        *year = 1 - (*year);
+
     if (t != NULL) *t = s;
 
     return cz;
 }
 
 static int
-typecast_parse_time(char* s, char** t, Py_ssize_t* len,
+typecast_parse_time(const char* s, const char** t, Py_ssize_t* len,
                      int* hh, int* mm, int* ss, int* us, int* tz)
 {
     int acc = -1, cz = 0;
@@ -123,6 +130,12 @@ typecast_parse_time(char* s, char** t, Py_ssize_t* len,
             else if (cz == 3) *us = acc;
             acc = -1; cz = 4;
             break;
+        case ' ':
+        case 'B':
+        case 'C':
+            /* Ignore the " BC" suffix, if passed -- it is handled
+             * when parsing the date portion. */
+            break;
         default:
             acc = (acc == -1 ? 0 : acc*10) + ((int)*s - (int)'0');
             if (cz == 3) usd += 1;
@@ -133,7 +146,9 @@ typecast_parse_time(char* s, char** t, Py_ssize_t* len,
     }
 
     if (acc != -1) {
-        if (cz == 2)      { *ss = acc; cz += 1; }
+        if (cz == 0)      { *hh = acc; cz += 1; }
+        else if (cz == 1) { *mm = acc; cz += 1; }
+        else if (cz == 2) { *ss = acc; cz += 1; }
         else if (cz == 3) { *us = acc; cz += 1; }
         else if (cz == 4) { tzhh = acc; cz += 1; }
         else if (cz == 5)   tzmm = acc;
@@ -167,7 +182,7 @@ typecast_parse_time(char* s, char** t, Py_ssize_t* len,
 
 /* a list of initializers, used to make the typecasters accessible anyway */
 #ifdef HAVE_PYDATETIME
-typecastObject_initlist typecast_pydatetime[] = {
+static typecastObject_initlist typecast_pydatetime[] = {
     {"PYDATETIME", typecast_DATETIME_types, typecast_PYDATETIME_cast},
     {"PYTIME", typecast_TIME_types, typecast_PYTIME_cast},
     {"PYDATE", typecast_DATE_types, typecast_PYDATE_cast},
@@ -178,7 +193,7 @@ typecastObject_initlist typecast_pydatetime[] = {
 
 /* a list of initializers, used to make the typecasters accessible anyway */
 #ifdef HAVE_MXDATETIME
-typecastObject_initlist typecast_mxdatetime[] = {
+static typecastObject_initlist typecast_mxdatetime[] = {
     {"MXDATETIME", typecast_DATETIME_types, typecast_MXDATE_cast},
     {"MXTIME", typecast_TIME_types, typecast_MXTIME_cast},
     {"MXDATE", typecast_DATE_types, typecast_MXDATE_cast},
@@ -508,8 +523,8 @@ typecast_from_python(PyObject *self, PyObject *args, PyObject *keywds)
 PyObject *
 typecast_from_c(typecastObject_initlist *type, PyObject *dict)
 {
-    PyObject *tuple, *base = NULL;
-    typecastObject *obj;
+    PyObject *name = NULL, *values = NULL, *base = NULL;
+    typecastObject *obj = NULL;
     Py_ssize_t i, len = 0;
 
     /* before doing anything else we look for the base */
@@ -518,32 +533,37 @@ typecast_from_c(typecastObject_initlist *type, PyObject *dict)
         base = PyDict_GetItemString(dict, type->base);
         if (!base) {
             PyErr_Format(Error, "typecast base not found: %s", type->base);
-            return NULL;
+            goto end;
         }
     }
 
+    name = PyString_FromString(type->name);
+    if (!name) goto end;
+
     while (type->values[len] != 0) len++;
 
-    tuple = PyTuple_New(len);
-    if (!tuple) return NULL;
+    values = PyTuple_New(len);
+    if (!values) goto end;
 
     for (i = 0; i < len ; i++) {
-        PyTuple_SET_ITEM(tuple, i, PyInt_FromLong(type->values[i]));
+        PyTuple_SET_ITEM(values, i, PyInt_FromLong(type->values[i]));
     }
 
-
-    obj = (typecastObject *)
-        typecast_new(PyString_FromString(type->name), tuple, NULL, base);
+    obj = (typecastObject *)typecast_new(name, values, NULL, base);
 
     if (obj) {
         obj->ccast = type->cast;
         obj->pcast = NULL;
     }
+
+ end:
+    Py_XDECREF(values);
+    Py_XDECREF(name);
     return (PyObject *)obj;
 }
 
 PyObject *
-typecast_cast(PyObject *obj, char *str, Py_ssize_t len, PyObject *curs)
+typecast_cast(PyObject *obj, const char *str, Py_ssize_t len, PyObject *curs)
 {
     PyObject *old, *res = NULL;
     typecastObject *self = (typecastObject *)obj;
