@@ -25,16 +25,16 @@
 
 /* the pointer to the datetime module API is initialized by the module init
    code, we just need to grab it */
-extern PyObject* pyDateTimeModuleP;
-extern PyObject *pyDateTypeP;
-extern PyObject *pyTimeTypeP;
-extern PyObject *pyDateTimeTypeP;
-extern PyObject *pyDeltaTypeP;
+extern HIDDEN PyObject* pyDateTimeModuleP;
+extern HIDDEN PyObject *pyDateTypeP;
+extern HIDDEN PyObject *pyTimeTypeP;
+extern HIDDEN PyObject *pyDateTimeTypeP;
+extern HIDDEN PyObject *pyDeltaTypeP;
 
 /** DATE - cast a date into a date python object **/
 
 static PyObject *
-typecast_PYDATE_cast(char *str, Py_ssize_t len, PyObject *curs)
+typecast_PYDATE_cast(const char *str, Py_ssize_t len, PyObject *curs)
 {
     PyObject* obj = NULL;
     int n, y=0, m=0, d=0;
@@ -58,9 +58,10 @@ typecast_PYDATE_cast(char *str, Py_ssize_t len, PyObject *curs)
                  n, len, y, m, d);
         if (n != 3) {
             PyErr_SetString(DataError, "unable to parse date");
+            return NULL;
         }
         else {
-        if (y > 9999) y = 9999;
+            if (y > 9999) y = 9999;
             obj = PyObject_CallFunction(pyDateTypeP, "iii", y, m, d);
         }
     }
@@ -70,12 +71,12 @@ typecast_PYDATE_cast(char *str, Py_ssize_t len, PyObject *curs)
 /** DATETIME - cast a timestamp into a datetime python object **/
 
 static PyObject *
-typecast_PYDATETIME_cast(char *str, Py_ssize_t len, PyObject *curs)
+typecast_PYDATETIME_cast(const char *str, Py_ssize_t len, PyObject *curs)
 {
     PyObject* obj = NULL;
     int n, y=0, m=0, d=0;
     int hh=0, mm=0, ss=0, us=0, tz=0;
-    char *tp = NULL;
+    const char *tp = NULL;
 
     if (str == NULL) {Py_INCREF(Py_None); return Py_None;}
 
@@ -98,6 +99,7 @@ typecast_PYDATETIME_cast(char *str, Py_ssize_t len, PyObject *curs)
                  tp, n, len, y, m, d);
         if (n != 3) {
             PyErr_SetString(DataError, "unable to parse date");
+            return NULL;
         }
 
         if (len > 0) {
@@ -108,6 +110,7 @@ typecast_PYDATETIME_cast(char *str, Py_ssize_t len, PyObject *curs)
                 n, len, hh, mm, ss, us, tz);
             if (n < 3 || n > 5) {
                 PyErr_SetString(DataError, "unable to parse time");
+                return NULL;
             }
         }
 
@@ -116,7 +119,7 @@ typecast_PYDATETIME_cast(char *str, Py_ssize_t len, PyObject *curs)
             ss -= 60;
         }
         if (y > 9999)
-        y = 9999;
+            y = 9999;
 
         if (n == 5 && ((cursorObject*)curs)->tzinfo_factory != Py_None) {
             /* we have a time zone, calculate minutes and create
@@ -144,7 +147,7 @@ typecast_PYDATETIME_cast(char *str, Py_ssize_t len, PyObject *curs)
 /** TIME - parse time into a time object **/
 
 static PyObject *
-typecast_PYTIME_cast(char *str, Py_ssize_t len, PyObject *curs)
+typecast_PYTIME_cast(const char *str, Py_ssize_t len, PyObject *curs)
 {
     PyObject* obj = NULL;
     int n, hh=0, mm=0, ss=0, us=0, tz=0;
@@ -158,6 +161,7 @@ typecast_PYTIME_cast(char *str, Py_ssize_t len, PyObject *curs)
 
     if (n < 3 || n > 5) {
         PyErr_SetString(DataError, "unable to parse time");
+        return NULL;
     }
     else {
         if (ss > 59) {
@@ -172,7 +176,7 @@ typecast_PYTIME_cast(char *str, Py_ssize_t len, PyObject *curs)
 /** INTERVAL - parse an interval into a timedelta object **/
 
 static PyObject *
-typecast_PYINTERVAL_cast(char *str, Py_ssize_t len, PyObject *curs)
+typecast_PYINTERVAL_cast(const char *str, Py_ssize_t len, PyObject *curs)
 {
     long years = 0, months = 0, days = 0;
     double hours = 0.0, minutes = 0.0, seconds = 0.0, hundredths = 0.0;
