@@ -23,16 +23,16 @@
 
 /* the pointer to the mxDateTime API is initialized by the module init code,
    we just need to grab it */
-extern mxDateTimeModule_APIObject *mxDateTimeP;
+extern HIDDEN mxDateTimeModule_APIObject *mxDateTimeP;
 
 /** DATE - cast a date into mx.DateTime python object **/
 
 static PyObject *
-typecast_MXDATE_cast(char *str, Py_ssize_t len, PyObject *curs)
+typecast_MXDATE_cast(const char *str, Py_ssize_t len, PyObject *curs)
 {
     int n, y=0, m=0, d=0;
     int hh=0, mm=0, ss=0, us=0, tz=0;
-    char *tp = NULL;
+    const char *tp = NULL;
 
     if (str == NULL) {Py_INCREF(Py_None); return Py_None;}
 
@@ -54,6 +54,7 @@ typecast_MXDATE_cast(char *str, Py_ssize_t len, PyObject *curs)
             " y = %d, m = %d, d = %d", tp, n, len, y, m, d);
     if (n != 3) {
         PyErr_SetString(DataError, "unable to parse date");
+        return NULL;
     }
 
     if (len > 0) {
@@ -62,8 +63,9 @@ typecast_MXDATE_cast(char *str, Py_ssize_t len, PyObject *curs)
             " len = " FORMAT_CODE_PY_SSIZE_T ","
             " hh = %d, mm = %d, ss = %d, us = %d, tz = %d",
             n, len, hh, mm, ss, us, tz);
-        if (n < 3 || n > 5) {
+        if (n != 0 && (n < 3 || n > 5)) {
             PyErr_SetString(DataError, "unable to parse time");
+            return NULL;
         }
     }
 
@@ -76,7 +78,7 @@ typecast_MXDATE_cast(char *str, Py_ssize_t len, PyObject *curs)
 /** TIME - parse time into an mx.DateTime object **/
 
 static PyObject *
-typecast_MXTIME_cast(char *str, Py_ssize_t len, PyObject *curs)
+typecast_MXTIME_cast(const char *str, Py_ssize_t len, PyObject *curs)
 {
     int n, hh=0, mm=0, ss=0, us=0, tz=0;
 
@@ -103,7 +105,7 @@ typecast_MXTIME_cast(char *str, Py_ssize_t len, PyObject *curs)
 /** INTERVAL - parse an interval into an mx.DateTimeDelta **/
 
 static PyObject *
-typecast_MXINTERVAL_cast(char *str, Py_ssize_t len, PyObject *curs)
+typecast_MXINTERVAL_cast(const char *str, Py_ssize_t len, PyObject *curs)
 {
     long years = 0, months = 0, days = 0, denominator = 1;
     double hours = 0.0, minutes = 0.0, seconds = 0.0, hundredths = 0.0;
